from django.utils.translation import ugettext as _
from django.db import models
from redactor.fields import RedactorField
from mptt.models import MPTTModel, TreeForeignKey
from django.utils.encoding import iri_to_uri
from django.core.urlresolvers import get_script_prefix
from django.utils.functional import lazy
from django.conf import settings
import os


class Menu(MPTTModel):
    name = models.CharField(_('Name'), max_length=255)
    url = models.CharField(_('URL'), max_length=255, blank=True, null=True)
    menu_id = models.SlugField(_('Menu ID'), blank=True, null=True)
    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
    published = models.BooleanField(_('Published'), default=True)

    class Meta:
        verbose_name = _('Menu')
        verbose_name_plural = _('Menus')

    def __unicode__(self):
        return self.name


def pages_templates():
    templates_path = os.path.join(settings.TEMPLATE_DIRS[0], 'pages')
    pages_templates = os.listdir(templates_path)
    pages_templates = tuple([(pt, pt) for pt in pages_templates])
    return pages_templates


class Page(models.Model):
    url = models.CharField(_('URL'), max_length=255)
    title = models.CharField(_('Page title'), max_length=255, blank=True, null=True)
    published = models.BooleanField(_('Published'), default=True)
    content = RedactorField(_('Content'), blank=True)
    template = models.CharField(_('Template name'), max_length=70,
        choices=lazy(pages_templates, tuple)(), blank=True, null=True)
    menu = models.ForeignKey(Menu, blank=True, null=True)

    def get_absolute_url(self):
        return iri_to_uri(get_script_prefix().rstrip('/') + self.url)

    class Meta:
        verbose_name = _('Page')
        verbose_name_plural = _('Pages')

    def __unicode__(self):
        return self.url



from django_zen.models import Menu
from django.utils.translation import ugettext as _
from django.utils.safestring import mark_safe
from django import template


register = template.Library()


class MenuTreeNode(template.Node):
    def __init__(self, template_nodes, menu_id):
        self.template_nodes = template_nodes
        self.menu_id = menu_id

    def _render_node(self, context, node):
        bits = []
        context.push()
        for child in node.get_children().filter(published=True):
            bits.append(self._render_node(context, child))
        context['menu_item'] = node
        context['children'] = mark_safe(''.join(bits))
        rendered = self.template_nodes.render(context)
        context.pop()
        return rendered

    def menu_items(self):
        try:
            root_menu = Menu.objects.get(menu_id=self.menu_id, published=True)
        except Menu.DoesNotExist:
            return []
        if root_menu:
            return root_menu.get_children()

    def render(self, context):
        roots = self.menu_items()
        bits = [self._render_node(context, node) for node in roots]
        return ''.join(bits)


@register.tag
def menu(parser, token):
    bits = token.contents.split()
    if len(bits) != 2:
        raise template.TemplateSyntaxError(_('%s tag requires a queryset') % bits[0])

    menu_id = template.Variable(bits[1])

    template_nodes = parser.parse(('endmenu',))
    parser.delete_first_token()

    return MenuTreeNode(template_nodes, menu_id)


@register.simple_tag
def active(request, name):
    if request.path == name:
        return ' active '
    return ''



from django.utils.translation import ugettext as _
from django.contrib import admin
from mptt.admin import FeinCMSModelAdmin
from django_zen.models import Menu, Page


class PagesAdmin(admin.ModelAdmin):
    list_display = ('url', 'title', 'published')

    def save_model(self, request, obj, form, change):
        obj.save()
        if form.cleaned_data['menu']:
            obj.menu.url = obj.url
            obj.menu.save()

    fieldsets = [
        (None, {'fields': ['url', 'title', 'template']}),
        (_('Advanced options'), {
            'classes': ('collapse',),
            'fields': ['menu', 'published']
        }),
        (_('Content'), {
            'classes': ('full-width',),
            'fields': ['content', ]
        }),
    ]


class MenuAdmin(FeinCMSModelAdmin):
    list_display = ('name', 'url', 'menu_id', 'published')

    fieldsets = [
        (None, {'fields': ['name', 'url', 'parent']}),
        (_('Menu settings'), {
            'classes': ('collapse',),
            'fields': ['menu_id', 'published']
        }),
    ]


admin.site.register(Menu, MenuAdmin)
admin.site.register(Page, PagesAdmin)



import os
from setuptools import setup, find_packages


def read_file(filename):
    path = os.path.abspath(os.path.dirname(__file__))
    filepath = os.path.join(path, filename)
    try:
        return open(filepath).read()
    except IOError:
        return ''


setup(
    name='django-zen',
    version='0.1.6',
    author='Vitaliy Korobkin',
    author_email='root@digitaldemiurge.me',
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/DigitalDemiurge/django-zen.git',
    install_requires=[
        'Django>=1.4',
        'django-mptt>=0.5',
        'feincms>=1.7.4',
        'django-wysiwyg-redactor==0.3.5',
    ],
    license='MIT license',
    description='Zen-Lightweight Django CMS',
    classifiers=[
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Framework :: Django',
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
    ],
    long_description=read_file('README.md'),
    test_suite="runtests.runtests",
    zip_safe=False,
)



from django.conf import settings
from django.shortcuts import get_object_or_404
from django.http import Http404, HttpResponsePermanentRedirect, HttpResponse
from django.views.decorators.csrf import csrf_protect
from django_zen.models import Page
from django.template import loader, RequestContext, Template, TemplateDoesNotExist
from django.utils.safestring import mark_safe


DEFAULT_TEMPLATE = 'pages/default.html'


def page(request, url):
    if not url.startswith('/'):
        url = '/' + url

    try:
        page = get_object_or_404(Page, url__exact=url, published=True)
    except Http404:
        if not url.endswith('/') and settings.APPEND_SLASH:
            url += '/'
            page = get_object_or_404(Page, url__exact=url, published=True)
            return HttpResponsePermanentRedirect('%s/' % request.path)
        else:
            raise
    return render_page(request, page)


@csrf_protect
def render_page(request, page):
    try:
        if page.template:
            template = loader.select_template(('pages/' + page.template, DEFAULT_TEMPLATE))
        else:
            template = loader.get_template(DEFAULT_TEMPLATE)
    except TemplateDoesNotExist:
        template = Template(
            "<html>" +
            "<head><title>Template not found</title></head>" +
            "<body>" +
            "<h1>Template {{ page.template }} not found</h1><br>" +
            "{{ page.content|safe }}" +
            "</body>" +
            "</html>"
        )

    page.title = mark_safe(page.title)
    page.content = mark_safe(page.content)

    c = RequestContext(request, {
        'page': page,
    })
    response = HttpResponse(template.render(c))
    return response



from django.conf.urls import patterns, url, include
from django_zen.views import page


urlpatterns = patterns('',
    url(r'^redactor/', include('redactor.urls')),
    url(r'^(?P<url>.*)$', page),
)
